#!/usr/bin/env ruby
# $Id: sign.rb,v 1.11 2014/09/10 12:05:04 rp Exp $
#
# PDFlib PLOP DS:
# Demonstrate digital signature features with optional signature visualization
#

require './PLOP'

# create a new PLOP object
plop = PLOP.new

# Basic signature option

sign_opts = 
	"sigtype=cms " +    # use sigtype=cades for PAdES signature 
        "ltv=try " +        #* use ltv=full to ensure LTV status */
	#when using PKCS11
        # "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
        "engine=builtin digitalid={filename=demorsa2048.p12} password={demo} "



# This is where input files live. Adjust as necessary.
searchpath = "../data"

# Set to 1 to enable signature visualization
visualize = 0
visfile = "signing_man_pdfa1a.pdf"

begin
    if (ARGV.length != 2 )
	raise "usage: sign <infile> <outfile>\n"
    end
    plop.set_option("searchpath=" + searchpath)

    if ((doc = plop.open_document(ARGV[0], "")) == -1)
	raise "ERROR: " + plop.get_errmsg()
    end
	# Basic signature options
	optlist = sign_opts
	
	if(visualize == 1)
		if ((visdoc = plop.open_document(visfile, "")) == -1)
		raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
		end
		# Add field option for visualization 
		optlist += "field ={visdoc=#{visdoc} rect={50 50 adapt adapt}} "
	end

    if (plop.prepare_signature(optlist) == -1)
		raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end

    if (plop.create_document(ARGV[1], "input=#{doc}") == -1)
		raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end
	
	plop.close_document(doc, "")
	if (visualize == 1)
		plop.close_document(visdoc, "")
	end

rescue PLOPException => pe
    print pe.backtrace.join("\n")
    printf("PLOP exception occurred in sign:\n")
    print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                        ": " + pe.get_errmsg + "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    plop.delete() if plop
end
